/*
** Module   : AbstractTTCollection.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------------Description----------------------------------------
** 001 ME  20200514 First version, stubs taken by converting the skeleton using FWD.
** 002 CA  20210113 Renamed Iiterator.next to Iiterator.next_, to follow NameConverter's rules.
**         20210113 Renamed clear() to clear_(), to follow NameConverter's rules.
** 003 ME  20210128 Keep only the default 4GL temp-table implementation.
** 004 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
**     CA  20210609 Updated INPUT/INPUT-OUTPUT parameters to the new approach, where they are explicitly 
**                  initialized at the method's execution, and not at the caller's arguments.
**     ME  20210928 Fix return value on clone method, defined out of the function block.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.oo.core.collections;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;

import com.goldencode.p2j.oo.core.Assert;
import com.goldencode.p2j.oo.core.collections.Icollection;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.persist.*;

import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.InternalEntry.Type;

/**
 * Business logic (converted to Java from the 4GL source code
 * in OpenEdge/Core/Collections/AbstractTTCollection.cls).
 */
@LegacyResource(resource = "OpenEdge.Core.Collections.AbstractTTCollection")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
public abstract class AbstractTtcollection extends BaseObject implements Icollection
{
   @LegacySignature(type = Type.PROPERTY, name = "Size")
   private integer size = TypeFactory.integer();

   private handle tableHandle = TypeFactory.handle();
   private handle fieldHandle = TypeFactory.handle();
   private handle queryHandle = TypeFactory.handle();

   public void __core_collections_AbstractTtcollection_execute__()
   {
      externalProcedure(AbstractTtcollection.class, AbstractTtcollection.this,
               new Block((Body) () ->
               {
                  onBlockLevel(Condition.ERROR, Action.THROW);
               }));
   }

   @LegacySignature(returns = "INTEGER", type = Type.GETTER, name = "Size")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public integer getSize()
   {
      return function(this, "Size", integer.class, new Block((Body) () -> {
         returnNormal(size);
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public void __core_collections_AbstractTtcollection_constructor__()
   {
      internalProcedure(AbstractTtcollection.class, this,
               "__core_collections_AbstractTtcollection_constructor__", new Block((Body) () ->
               {
                  __lang_BaseObject_constructor__();
               }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "HANDLE", mode = "INPUT"),
            @LegacyParameter(name = "p3", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected void __core_collections_AbstractTtcollection_constructor__(
            final object<? extends Icollection> _p1, final handle _p2, final character _p3)
   {
      internalProcedure(AbstractTtcollection.class, this,
               "__core_collections_AbstractTtcollection_constructor__", new Block((Body) () ->
               {
                  __core_collections_AbstractTtcollection_constructor__(_p2, _p3);
                  addAll(_p1);
               }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = {
            @LegacyParameter(name = "p1", type = "HANDLE", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected void __core_collections_AbstractTtcollection_constructor__(final handle _p1,
            final character _p2)
   {
      internalProcedure(AbstractTtcollection.class, this,
               "__core_collections_AbstractTtcollection_constructor__", new Block((Body) () -> {
                  __core_collections_AbstractTtcollection_constructor__(_p1, _p1.unwrapTempTable()
                           .defaultBufferHandle().unwrapBuffer().bufferField(_p2));
               }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = {
            @LegacyParameter(name = "p1", type = "HANDLE", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "HANDLE", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected void __core_collections_AbstractTtcollection_constructor__(final handle _p1,
            final handle _p2)
   {
      handle p1 = TypeFactory.initInput(_p1);
      handle p2 = TypeFactory.initInput(_p2);

      internalProcedure(AbstractTtcollection.class, this,
               "__core_collections_AbstractTtcollection_constructor__", new Block((Body) () ->
               {
                  __lang_BaseObject_constructor__();
                  size.assign(0);

                  tableHandle.assign(p1.unwrapTempTable());
                  fieldHandle.assign(p2.unwrapBufferField());
               }));
   }

   @LegacySignature(type = Type.DESTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public void __core_collections_AbstractTtcollection_destructor__()
   {
      internalProcedure(AbstractTtcollection.class, this,
               "__core_collections_AbstractTtcollection_destructor__", new Block((Body) () ->
               {
                  if (queryHandle._isValid())
                     HandleOps.delete(queryHandle);
               }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Add", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical add(final object<? extends _BaseObject_> _p1)
   {
      object<? extends _BaseObject_> p1 = TypeFactory.initInput(_p1);

      return function(this, "Add", logical.class, new Block((Body) () -> {

         if (p1._isValid())
         {
               Buffer buffer = tableHandle.unwrapTempTable().defaultBufferHandle().unwrapBuffer();
               buffer.bufferCreate();
               fieldHandle.unwrapBufferField().changeValue(p1);
               buffer.bufferRelease();

               size.assign(size.intValue() + 1);
               
               returnNormal(true);
         }

         returnNormal(false);

      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "AddArray", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", extent = -1, qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical addArray(final object<? extends _BaseObject_>[] _p1)
   {
      object<? extends _BaseObject_> p1[] = TypeFactory.initInput(_p1);

      return function(this, "AddArray", logical.class, new Block((Body) () -> {
         if (p1.length > 0)
         {
            for (int i = 0; i < p1.length; i++)
            {
               add(p1[i]);
            }
         }

         returnNormal(true);
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "AddAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical addAll(final object<? extends Icollection> _p1)
   {
      object<? extends Icollection> p1 = TypeFactory.initInput(_p1);

      return function(this, "AddAll", logical.class, new Block((Body) () -> {
         if (getLegacyClass().ref()
                  .legacyEquals(p1.ref().getLegacyClass()).booleanValue())
         {
            p1.ref().toTable(new TableParameter(tableHandle, true));
            size.assign(size.intValue() + p1.ref().getSize().intValue());
         }
         else
         {
            Iiterator it = p1.ref().iterator().ref();

            while (it.hasNext().booleanValue())
            {
               add(it.next_());
            }
         }

         returnNormal(true);
      }));
   }

   @LegacySignature(type = Type.METHOD, name = "Clear")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public void clear_()
   {
      internalProcedure(AbstractTtcollection.class, this, "Clear", new Block((Body) () ->
      {
         tableHandle.unwrapTempTable().deleteAll();
         size.assign(0);
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Contains", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical contains(final object<? extends _BaseObject_> _p1)
   {
      object<? extends _BaseObject_> p1 = TypeFactory.initInput(_p1);

      return function(this, "Contains", logical.class, new Block((Body) () -> {
         findBufferUseObject(p1);
         returnNormal(tableHandle.unwrapTempTable().defaultBufferHandle().unwrapBuffer()
                  .available());
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "ContainsAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical containsAll(final object<? extends Icollection> _p1)
   {
      object<? extends Icollection> p1 = TypeFactory.initInput(_p1);

      return function(this, "ContainsAll", logical.class, new Block((Body) () -> {
         Assert.notNull(p1, new character("Collection to check"));

         if (p1.ref().isEmpty().booleanValue())
            returnNormal(true);

         // this is handled like a set not collection 
         if (p1.ref().getSize().intValue() > size.intValue())
            returnNormal(false);

         Iiterator it = p1.ref().iterator().ref();

         while (it.hasNext().booleanValue())
         {
            if (!contains(it.next_()).booleanValue())
               returnNormal(false);
         }

         returnNormal(true);
      }));
   }

   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Iterator", qualified = "openedge.core.collections.iiterator")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends com.goldencode.p2j.oo.core.collections.Iiterator> iterator()
   {
      return function(this, "Iterator", object.class, new Block((Body) () -> {
         returnNormal(ObjectOps.newInstance(LegacyIterator.class, "III", new object(this),
                  tableHandle, fieldHandle.unwrap().name()));
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "IsEmpty")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical isEmpty()
   {
      return function(this, "IsEmpty", logical.class, new Block((Body) () -> {
         returnNormal(!tableHandle.unwrapTempTable().hasRecords().booleanValue());
      }));
   }

   @LegacySignature(type = Type.METHOD, name = "ToTable", parameters = {
            @LegacyParameter(name = "p1", type = "HANDLE", mode = "OUTPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public void toTable(final TableParameter _p1)
   {
      handle p1 = new handle();

      internalProcedure(AbstractTtcollection.class, this, "ToTable", new Block((Body) () ->
      {
         TemporaryBuffer.createDynamicTable(_p1, p1, false, true);
         p1.assign(tableHandle);
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Remove", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical remove(final object<? extends _BaseObject_> _p1)
   {
      object<? extends _BaseObject_> p1 = TypeFactory.initInput(_p1);

      return function(this, "Remove", logical.class, new Block((Body) () -> {
         boolean result = false;

         if (p1._isValid())
         {
               findBufferUseObject(p1);
               Buffer buffer = tableHandle.unwrapTempTable().defaultBufferHandle().unwrapBuffer();
               if (buffer._available())
               {
                  buffer.deleteRecord();
                  result = true;
               }
         }

         if (result)
            size.assign(size.intValue() - 1);

         returnNormal(result);
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "RemoveAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical removeAll(final object<? extends Icollection> _p1)
   {
      object<? extends Icollection> p1 = TypeFactory.initInput(_p1);

      return function(this, "RemoveAll", logical.class, new Block((Body) () -> {
         boolean result = false;
         Iiterator it = p1.ref().iterator().ref();

         while (it.hasNext().booleanValue())
         {
            object<? extends _BaseObject_> obj = it.next_();
            while (remove(obj).booleanValue())
               result = true;
         }

         returnNormal(result);
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "RetainAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical retainAll(final object<? extends Icollection> _p1)
   {
      object<? extends Icollection> p1 = TypeFactory.initInput(_p1);

      return function(this, "RetainAll", logical.class, new Block((Body) () -> {
         boolean result = false;
         Iiterator it = iterator().ref();

         while (it.hasNext().booleanValue())
         {
            object<? extends _BaseObject_> obj = it.next_();

            if (!p1.ref().contains(obj).booleanValue())
            {
               while (remove(obj).booleanValue())
                  result = true;
            }
         }

         returnNormal(result);
      }));
   }

   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "ToArray", extent = -1, qualified = "progress.lang.object")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends com.goldencode.p2j.oo.lang._BaseObject_>[] toArray()
   {
      object oArr[][] = { TypeFactory.objectExtent(BaseObject.class) };
      return extentFunction(this, "ToArray", object.class, new Block((Body) () -> {
         
         if (isEmpty().booleanValue())
            returnExtentNormal(new object[0]);
         else
         {
            Iiterator it = iterator().ref();
            int i = 0;

            oArr[0] = ArrayAssigner.resize(oArr[0], getSize().intValue());
            while (it.hasNext().booleanValue())
            {
               oArr[0][i++].assign(it.next_());
            }
            returnExtentNormal(oArr[0]);
         }
      }));
   }

   @LegacySignature(type = Type.METHOD, name = "FindBufferUseObject", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   protected abstract void findBufferUseObject(final object<? extends _BaseObject_> _p1);

   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Clone", qualified = "progress.lang.object")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   @Override
   public object<? extends com.goldencode.p2j.oo.lang._BaseObject_> clone()
   {
      object<? extends Icollection> clone = TypeFactory.object(Icollection.class);
      
      return function(this, "Clone", object.class, new Block((Body) () -> {
         clone.assign(ObjectOps.newInstance(getClass()));
         cloneElements(clone);
         returnNormal(clone);
      }));
   }

   @LegacySignature(type = Type.METHOD, name = "CloneElements", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected void cloneElements(final object<? extends Icollection> _p1)
   {
      object<? extends Icollection> p1 = TypeFactory.initInput(_p1);

      internalProcedure(AbstractTtcollection.class, this, "CloneElements", new Block((Body) () ->
      {
         Iiterator it = iterator().ref();
         while (it.hasNext().booleanValue())
            p1.ref().add(it.next_().ref().clone());
      }));
   }

   @LegacySignature(type = Type.METHOD, name = "Resize")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected void resize()
   {
      internalProcedure(AbstractTtcollection.class, this, "Resize", new Block((Body) () ->
      {
         if (!queryHandle._isValid()) {
            QueryWrapper.createQuery(queryHandle);
            queryHandle.unwrapBufferCollection().setBuffers(tableHandle.unwrapTempTable().defaultBufferHandle());
            queryHandle.unwrapQuery().prepare(String.format("preselect each %s", tableHandle.unwrap().name().getValue()));
         }
         
         queryHandle.unwrapQuery().queryOpen();
         size.assign(queryHandle.unwrapQuery().getNumResults());
         queryHandle.unwrapQuery().close();
      }));
   }
}
