/*
** Module   : MapBackedCollection.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------------Description----------------------------------------
** 001 ME  20200514 First version, stubs taken by converting the skeleton using FWD.
** 002 CA  20210113 Renamed Iiterator.next to Iiterator.next_, to follow NameConverter's rules.
**                  Renamed clear() to clear_(), to follow NameConverter's rules.
** 003 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
**     CA  20210609 Updated INPUT/INPUT-OUTPUT parameters to the new approach, where they are explicitly 
**                  initialized at the method's execution, and not at the caller's arguments.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.oo.core.collections;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;

import com.goldencode.p2j.oo.core.collections.Icollection;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.oo.core.collections.Imap;
import com.goldencode.p2j.oo.core.system.UnsupportedOperationError;
import com.goldencode.p2j.oo.lang._BaseObject_;
import com.goldencode.p2j.persist.*;

import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.InternalEntry.Type;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;

/**
 * Business logic (converted to Java from the 4GL source code
 * in skeleton/oo4gl/OpenEdge/Core/Collections/MapBackedCollection.cls).
 */
@LegacyResource(resource = "OpenEdge.Core.Collections.MapBackedCollection")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_PARTIAL)
public abstract class MapBackedCollection extends BaseObject implements Icollection
{
   @LegacySignature(type = Type.PROPERTY, name = "OwningMap")
   private object<? extends Imap> owningMap = TypeFactory.object(Imap.class);

   @LegacySignature(type = Type.PROPERTY, name = "OwningBuffer")
   private handle owningBuffer = TypeFactory.handle();

   @LegacySignature(type = Type.PROPERTY, name = "KeyField")
   private handle keyField = TypeFactory.handle();

   private String keyFieldName;

   private Map<object<? extends _BaseObject_>, object<? extends _BaseObject_>> objectsMap;

   public void __core_collections_MapBackedCollection_execute__()
   {
      externalProcedure(MapBackedCollection.class, MapBackedCollection.this,
               new Block((Body) () ->
               {
                  onBlockLevel(Condition.ERROR, Action.THROW);
               }));
   }

   @LegacySignature(returns = "OBJECT", qualified = "OpenEdge.Core.Collections.IMap", type = Type.GETTER, name = "OwningMap")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected object<? extends Imap> getOwningMap()
   {
      return function(MapBackedCollection.class, this, "OwningMap", object.class,
               new Block((Body) () ->
               {
                  returnNormal(owningMap);
               }));
   }

   @LegacySignature(returns = "HANDLE", type = Type.GETTER, name = "OwningBuffer")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected handle getOwningBuffer()
   {
      return function(MapBackedCollection.class, this, "OwningBuffer", handle.class,
               new Block((Body) () ->
               {
                  returnNormal(owningBuffer);
               }));
   }

   @LegacySignature(returns = "HANDLE", type = Type.GETTER, name = "KeyField")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected handle getKeyField()
   {
      return function(MapBackedCollection.class, this, "KeyField", handle.class,
               new Block((Body) () ->
               {
                  returnNormal(keyField);
               }));
   }

   @LegacySignature(returns = "INTEGER", type = Type.GETTER, name = "Size")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public integer getSize()
   {
      return function(MapBackedCollection.class, this, "Size", integer.class,
               new Block((Body) () ->
               {
                  returnNormal(owningMap.ref().getSize());
               }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = {
            @LegacyParameter(name = "poMap", type = "OBJECT", qualified = "openedge.core.collections.imap", mode = "INPUT"),
            @LegacyParameter(name = "phTT", type = "HANDLE", mode = "INPUT"),
            @LegacyParameter(name = "pcValuefield", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public void __core_collections_MapBackedCollection_constructor__(
            final object<? extends Imap> _poMap, final handle _phTt,
            final character _pcValuefield)
   {
      object<? extends Imap> poMap = TypeFactory.initInput(_poMap);
      handle phTt = TypeFactory.initInput(_phTt);
      character pcValuefield = TypeFactory.initInput(_pcValuefield);

      internalProcedure(MapBackedCollection.class, this,
               "__core_collections_MapBackedCollection_constructor__", new Block((Body) () ->
               {
                  __lang_BaseObject_constructor__();
                  keyFieldName = pcValuefield.getValue();
                  owningMap.assign(poMap);

                  // default 4GL temp-table implementation
                  if (phTt.isType(LegacyResource.TEMP_TABLE))
                  {
                     this.owningBuffer.assign(phTt);
                     this.keyField.assign(phTt.unwrapTempTable().defaultBufferHandle()
                              .unwrapBuffer().bufferField(pcValuefield));
                     objectsMap = Collections.EMPTY_MAP;
                  }
                  // java map implementation
                  else
                  {
                     Object resource = phTt.get();

                     if (resource instanceof Map)
                        objectsMap = (Map<object<? extends _BaseObject_>, object<? extends _BaseObject_>>) resource;
                  }

               }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Add", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical add(final object<? extends _BaseObject_> _p1)
   {
      return function(MapBackedCollection.class, this, "Add", logical.class,
               new Block((Body) () ->
               {
                  undoThrow(ObjectOps.newInstance(UnsupportedOperationError.class, "II", "Add",
                           getLegacyClass().ref().getTypeName()));
               }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "AddAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical addAll(final object<? extends Icollection> _p1)
   {
      return function(MapBackedCollection.class, this, "AddAll", logical.class,
               new Block((Body) () ->
               {
                  undoThrow(ObjectOps.newInstance(UnsupportedOperationError.class, "II", "AddAll",
                           getLegacyClass().ref().getTypeName()));
               }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "AddArray", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", extent = -1, qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical addArray(final object<? extends _BaseObject_>[] _p1)
   {
      return function(MapBackedCollection.class, this, "AddArray", logical.class,
               new Block((Body) () ->
               {
                  undoThrow(ObjectOps.newInstance(UnsupportedOperationError.class, "II",
                           "AddArray", getLegacyClass().ref().getTypeName()));
               }));
   }

   @LegacySignature(type = Type.METHOD, name = "Clear")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public void clear_()
   {
      internalProcedure(MapBackedCollection.class, this, "Clear", new Block((Body) () ->
      {
         owningMap.ref().clear_();
      }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "IsEmpty")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public logical isEmpty()
   {
      return function(MapBackedCollection.class, this, "IsEmpty", logical.class,
               new Block((Body) () ->
               {
                  returnNormal(owningMap.ref().isEmpty());
               }));
   }

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Contains", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   public abstract logical contains(final object<? extends _BaseObject_> _p1);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "ContainsAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   public abstract logical containsAll(final object<? extends Icollection> _p1);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Remove", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   public abstract logical remove(final object<? extends _BaseObject_> _p1);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "RemoveAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   public abstract logical removeAll(final object<? extends Icollection> _p1);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "RetainAll", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT") })
   public abstract logical retainAll(final object<? extends Icollection> _p1);

   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Iterator", qualified = "openedge.core.collections.iiterator")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends com.goldencode.p2j.oo.core.collections.Iiterator> iterator()
   {
      return function(MapBackedCollection.class, this, "Iterator", object.class,
               new Block((Body) () ->
               {
                  if (objectsMap != Collections.EMPTY_MAP)
                  {
                     returnNormal(
                              ObjectOps.newInstance(LegacyIterator.class, "III", new object(this),
                                       new handle(new TransparentWrapper(getCollection())), ""));
                  }
                  else
                  {
                     returnNormal(ObjectOps.newInstance(LegacyIterator.class, "III",
                              new object(this), owningBuffer, keyFieldName));
                  }
               }));
   }

   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "ToArray", extent = -1, qualified = "progress.lang.object")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends com.goldencode.p2j.oo.lang._BaseObject_>[] toArray()
   {
      object oArr[][] = { TypeFactory.objectExtent(BaseObject.class) };

      return extentFunction(MapBackedCollection.class, this, "ToArray", object.class,
               new Block((Body) () ->
               {
                  if (objectsMap != Collections.EMPTY_MAP)
                  {
                     // if we already have the collection
                     returnExtentNormal(getCollection().toArray(new object[objectsMap.size()]));
                  }
                  else
                  {
                     // the 4GL way
                     if (isEmpty().booleanValue())
                        returnExtentNormal(new object[0]);
                     else
                     {
                        oArr[0] = ArrayAssigner.resize(oArr[0], getSize().intValue());
                        Iiterator it = iterator().ref();
                        int i = 0;

                        while (it.hasNext().booleanValue())
                        {
                           oArr[0][i++].assign(it.next_());
                        }
                        returnExtentNormal(oArr[0]);
                     }
                  }
               }));
   }

   @LegacySignature(type = Type.METHOD, name = "ToTable", parameters = {
            @LegacyParameter(name = "p1", type = "HANDLE", mode = "OUTPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_PARTIAL)
   public void toTable(final TableParameter _p1)
   {
      handle p1 = new handle();

      internalProcedure(MapBackedCollection.class, this, "ToTable", new Block((Body) () ->
      {
         if (objectsMap != Collections.EMPTY_MAP)
         {
            TemporaryBuffer.createDynamicTable(_p1, p1, false, true);
            p1.assign(owningBuffer);
         }
         // TODO: add object to temp-table if java map is used
         else
         {
         }

      }));
   }

   protected Map<object<? extends _BaseObject_>, object<? extends _BaseObject_>> getMap()
   {
      return objectsMap;
   }

   protected Collection<object<? extends _BaseObject_>> getCollection()
   {
      if ("KeyRef".equalsIgnoreCase(keyFieldName))
         return objectsMap.keySet();
      else
         return objectsMap.values();
   }

}
