/*
** Module   : MediaLinkEntity.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2020-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 MP  20200602 First version, stubs taken by converting the skeleton using FWD.
** 002 ME  20201126 Implement all methods as per OE12.2.
** 
** 003 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
**     ME  20210929 Fix return object on deserialize, use enum literals in substitute.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.oo.core.Assert;
import com.goldencode.p2j.oo.core.SerializationFormatEnum;
import com.goldencode.p2j.oo.json.objectmodel.JsonObject;
import com.goldencode.p2j.oo.json.objectmodel.ObjectModelParser;
import com.goldencode.p2j.oo.lang.*;

import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps._isEqual;
import static com.goldencode.p2j.util.InternalEntry.Type;

/**
 * Business logic (converted to Java from the 4GL source code
 * in OpenEdge/Net/MediaLinkEntity.cls).
 */
@LegacyResource(resource = "OpenEdge.Net.MediaLinkEntity")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class MediaLinkEntity
extends BaseObject
{
   @LegacySignature(type = Type.PROPERTY, name = "ReadUrl")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   private character readUrl = TypeFactory.character();

   @LegacySignature(type = Type.PROPERTY, name = "EditUrl")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   private character editUrl = TypeFactory.character();

   @LegacySignature(type = Type.PROPERTY, name = "ResourceName")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   private character resourceName = TypeFactory.character();

   public void __net_MediaLinkEntity_execute__()
   {
      externalProcedure(MediaLinkEntity.class, MediaLinkEntity.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
      }));
   }

   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "ReadUrl")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public character getReadUrl()
   {
      return function(MediaLinkEntity.class, this, "ReadUrl", character.class, new Block((Body) () -> 
      {
         returnNormal(readUrl);
      }));
   }

   @LegacySignature(type = Type.SETTER, name = "ReadUrl", parameters = 
   {
      @LegacyParameter(name = "var", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void setReadUrl(final character _var)
   {
      character var = TypeFactory.initInput(_var);
      
      internalProcedure(MediaLinkEntity.class, this, "ReadUrl", new Block((Body) () -> 
      {
         Assert.notNull(var, new character("Media resource read URL"));
         
         readUrl.assign(var);
      }));
   }

   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "EditUrl")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public character getEditUrl()
   {
      return function(MediaLinkEntity.class, this, "EditUrl", character.class, new Block((Body) () -> 
      {
         returnNormal(editUrl);
      }));
   }

   @LegacySignature(type = Type.SETTER, name = "EditUrl", parameters = 
   {
      @LegacyParameter(name = "var", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void setEditUrl(final character _var)
   {
      character var = TypeFactory.initInput(_var);
      
      internalProcedure(MediaLinkEntity.class, this, "EditUrl", new Block((Body) () -> 
      {
         Assert.notNull(var, new character("Media resource edit URL"));
         
         editUrl.assign(var);
      }));
   }

   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "ResourceName")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public character getResourceName()
   {
      return function(MediaLinkEntity.class, this, "ResourceName", character.class, new Block((Body) () -> 
      {
         returnNormal(resourceName);
      }));
   }

   @LegacySignature(type = Type.SETTER, name = "ResourceName", parameters = 
   {
      @LegacyParameter(name = "var", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void setResourceName(final character _var)
   {
      character var = TypeFactory.initInput(_var);
      
      internalProcedure(MediaLinkEntity.class, this, "ResourceName", new Block((Body) () -> 
      {
         Assert.notNull(var, new character("Media resource name"));
         
         resourceName.assign(var);
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = 
   {
      @LegacyParameter(name = "pcReadUrl", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_MediaLinkEntity_constructor__(final character _pcReadUrl)
   {
      internalProcedure(MediaLinkEntity.class, this, "__net_MediaLinkEntity_constructor__", new Block((Body) () -> 
      {
         __lang_BaseObject_constructor__();
         setReadUrl(_pcReadUrl);
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = 
   {
      @LegacyParameter(name = "pcReadUrl", type = "CHARACTER", mode = "INPUT"),
      @LegacyParameter(name = "pcEditUrl", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_MediaLinkEntity_constructor__(final character _pcReadUrl, final character _pcEditUrl)
   {
      internalProcedure(MediaLinkEntity.class, this, "__net_MediaLinkEntity_constructor__", new Block((Body) () -> 
      {
         __net_MediaLinkEntity_constructor__(_pcReadUrl);
         setEditUrl(_pcEditUrl);
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = 
   {
      @LegacyParameter(name = "poReadUrl", type = "OBJECT", qualified = "openedge.net.uri", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_MediaLinkEntity_constructor__(final object<? extends com.goldencode.p2j.oo.net.Uri> _poReadUrl)
   {
      object<? extends com.goldencode.p2j.oo.net.Uri> poReadUrl = TypeFactory.initInput(_poReadUrl);
      
      internalProcedure(MediaLinkEntity.class, this, "__net_MediaLinkEntity_constructor__", new Block((Body) () -> 
      {
         Assert.notNull(poReadUrl, new character("Read URI"));
         
         __net_MediaLinkEntity_constructor__(poReadUrl.ref().toLegacyString());
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR, parameters = 
   {
      @LegacyParameter(name = "poReadUrl", type = "OBJECT", qualified = "openedge.net.uri", mode = "INPUT"),
      @LegacyParameter(name = "poEditUrl", type = "OBJECT", qualified = "openedge.net.uri", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_MediaLinkEntity_constructor__(final object<? extends com.goldencode.p2j.oo.net.Uri> _poReadUrl, final object<? extends com.goldencode.p2j.oo.net.Uri> _poEditUrl)
   {
      object<? extends com.goldencode.p2j.oo.net.Uri> poReadUrl = TypeFactory.initInput(_poReadUrl);
      object<? extends com.goldencode.p2j.oo.net.Uri> poEditUrl = TypeFactory.initInput(_poEditUrl);
      
      internalProcedure(MediaLinkEntity.class, this, "__net_MediaLinkEntity_constructor__", new Block((Body) () -> 
      {
         Assert.notNull(poReadUrl, new character("Read URI"));
         Assert.notNull(poEditUrl, new character("Edit URI"));
         
         __net_MediaLinkEntity_constructor__(poReadUrl.ref().toLegacyString(), poEditUrl.ref().toLegacyString());
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_MediaLinkEntity_constructor__()
   {
      internalProcedure(MediaLinkEntity.class, this, "__net_MediaLinkEntity_constructor__", new Block((Body) () -> 
      {
         __lang_BaseObject_constructor__();
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static void __net_MediaLinkEntity_constructor__static__()
   {
      externalProcedure(MediaLinkEntity.class, new Block((Body) () ->
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
      }));
   }
   
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Deserialize", qualified = "openedge.net.medialinkentity", parameters = 
   {
      @LegacyParameter(name = "pcMLE", type = "LONGCHAR", mode = "INPUT"),
      @LegacyParameter(name = "poSerializedAs", type = "OBJECT", qualified = "openedge.core.serializationformatenum", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public static object<? extends MediaLinkEntity> deserialize(final longchar _pcMle, final object<? extends com.goldencode.p2j.oo.core.SerializationFormatEnum> _poSerializedAs)
   {
      longchar pcMle = TypeFactory.initInput(_pcMle);
      object<? extends com.goldencode.p2j.oo.core.SerializationFormatEnum> poSerializedAs = TypeFactory.initInput(_poSerializedAs);
      object<MediaLinkEntity> mle = TypeFactory.object(MediaLinkEntity.class);
      
      return function(MediaLinkEntity.class, "Deserialize", object.class, new Block((Body) () -> 
      {
         Assert.notNull(poSerializedAs, new character("MLE serialization type"));
         Assert.notNullOrEmpty(pcMle, new character("Serialized MLE"));

         if (!_isEqual(poSerializedAs, SerializationFormatEnum.json)
                  && !_isEqual(poSerializedAs, SerializationFormatEnum.quotedJson))
         {
            undoThrow(AppError.newInstance(
                     TextOps.substitute("Unsupported MLE serialization type &1",
                              poSerializedAs.ref().toLegacyString()),
                     new integer(0)));
         }
         
         mle.assign(ObjectOps.newInstance(MediaLinkEntity.class));
         
         object<ObjectModelParser> parser = ObjectOps.newInstance(ObjectModelParser.class);
         object<JsonObject> jsonObj = ObjectOps.cast(parser.ref().parse(pcMle), JsonObject.class);

         character prop = new character("src");
         
         if (jsonObj.ref().has(prop).booleanValue())
         {
            mle.ref().setReadUrl(jsonObj.ref().getCharacter(prop));
            
            prop.assign("name");
            
            if (jsonObj.ref().has(prop).booleanValue())
            {
               mle.ref().setResourceName(jsonObj.ref().getCharacter(prop));
            }
         }
         
         prop.assign("edit-media");
         
         if (jsonObj.ref().has(prop).booleanValue())
         {
            mle.ref().setEditUrl(jsonObj.ref().getCharacter(prop));
         }
         
         returnNormal(mle);
      }));
   }

   @LegacySignature(returns = "LONGCHAR", type = Type.METHOD, name = "Serialize", parameters = {
            @LegacyParameter(name = "poSerializedAs", type = "OBJECT", qualified = "openedge.core.serializationformatenum", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public longchar serialize(
            final object<? extends com.goldencode.p2j.oo.core.SerializationFormatEnum> _poSerializedAs)
   {
      object<? extends com.goldencode.p2j.oo.core.SerializationFormatEnum> poSerializedAs = TypeFactory
               .initInput(_poSerializedAs);
      longchar payload = TypeFactory.longchar();

      return function(MediaLinkEntity.class, this, "Serialize", longchar.class,
               new Block((Body) () -> {
                  Assert.notNull(poSerializedAs, new character("MLE serialization type"));

                  if (TextOps.isEmpty(readUrl) && TextOps.isEmpty(editUrl))
                  {
                     undoThrow(AppError
                              .newInstance("Either a ReadUrl or EditUrl must be specified", 0));
                  }

                  if (!_isEqual(poSerializedAs, SerializationFormatEnum.json)
                           && !_isEqual(poSerializedAs, SerializationFormatEnum.quotedJson))
                  {
                     undoThrow(AppError.newInstance(
                              TextOps.substitute("Unsupported MLE serialization type &1",
                                       poSerializedAs.ref().toLegacyString()),
                              new integer(0)));
                  }

                  object<JsonObject> jsonObj = ObjectOps.newInstance(JsonObject.class);

                  if (!TextOps.isEmpty(readUrl))
                  {
                     jsonObj.ref().add(new character("src"), readUrl);

                     if (!TextOps.isEmpty(resourceName))
                     {
                        jsonObj.ref().add(new character("name"), resourceName);
                     }
                  }

                  if (!TextOps.isEmpty(editUrl))
                  {
                     jsonObj.ref().add(new character("edit-media"), editUrl);
                  }

                  payload.fixCodePage("utf-8");
                  payload.assign(jsonObj.ref().getJsonText());

                  if (_isEqual(poSerializedAs, SerializationFormatEnum.quotedJson)) 
                  {
                     payload.assign(character.quoter(payload));
                  }
                  
                  returnNormal(payload);

               }));
   }
   
}
