/*
** Module   : MimeTypeHelper.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 MP  20200401 First version, stubs taken by converting the skeleton using FWD.
** 002 ME  20201019 Remove dependency to StringUtils.
** 003 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.oo.core.Assert;
import com.goldencode.p2j.oo.lang.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.InternalEntry.Type;

/**
 * Business logic (converted to Java from the 4GL source code
 * in OpenEdge/Net/MessagePart.cls).
 */
@LegacyResource(resource = "OpenEdge.Net.MimeTypeHelper")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
public class MimeTypeHelper extends BaseObject
{

   /**
    * Execute method
    */
   public void __net_MimeTypeHelper_execute__()
   {
      externalProcedure(MimeTypeHelper.this, new Block((Body) () ->
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
         }
      }));
   }

   /**
    * Constructor.
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   private void __net_MimeTypeHelper_constructor__()
   {
      internalProcedure(MimeTypeHelper.class, this, "__net_MimeTypeHelper_constructor__",
               new Block((Body) () ->
               {
                  __lang_BaseObject_constructor__();
               }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static void __net_MimeTypeHelper_constructor__static__()
   {
      externalProcedure(MimeTypeHelper.class, new Block((Body) () ->
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
      }));
   }

   /**
    * Strips the vendor prefix and any parmeters from the input type, and 
    *  returns the type and subtype in an array
    *
    * @param _p1 the mime type
    * @return an array of the type [1] and subtype [2]
    */
   @LegacySignature(returns = "CHARACTER", type = Type.METHOD, name = "SplitType", extent = 2, parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static character[] splitType(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);
      ObjectOps.load(MimeTypeHelper.class);
      character[][] rsp = { TypeFactory.characterExtent(2) };

      return extentFunction(MimeTypeHelper.class, "SplitType", character.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("MIME type"));

                  String[] entries = splitType(p1.getValue(), 2);

                  rsp[0][0].assign(entries[0]);
                  rsp[0][1].assign(entries[1]);

                  returnExtentNormal(rsp[0]);

               }));
   }

   /**
    * Indicates if a given MIME type is a form encoded type
    *
    * @param _p1 the mime type
    * @return TRUE if the input type is valid and form-encoded.
    */
   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "IsFormEncoded", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static logical isFormEncoded(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);
      ObjectOps.load(MimeTypeHelper.class);

      return function(MimeTypeHelper.class, "IsFormEncoded", logical.class, new Block((Body) () ->
      {

         if (p1.isUnknown() || TextOps.isEmpty(p1.getValue()))
         {
            returnNormal(new logical(false));
         }

         String[] entries = splitType(p1.getValue(), -1);

         if (entries.length != 2)
         {
            returnNormal(new logical(false));
         }

         returnNormal(new logical((entries[0].equalsIgnoreCase("application")
                  && entries[1].equalsIgnoreCase("x-www-form-urlencoded"))
                  || (entries[0].equalsIgnoreCase("multipart")
                           && entries[1].equalsIgnoreCase("form-data"))));

      }));
   }

   /**
    * Indicates if a given MIME type is a multipart type
    *
    * @param _p1 the mime type
    * @return TRUE if the input type is valid and multipart.
    */
   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "IsMultipart", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static logical isMultipart(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);
      ObjectOps.load(MimeTypeHelper.class);

      return function(MimeTypeHelper.class, "IsMultipart", logical.class, new Block((Body) () ->
      {
         if (p1.isUnknown() || TextOps.isEmpty(p1.getValue()))
         {
            returnNormal(new logical(false));
         }

         String[] entries = splitType(p1.getValue(), -1);

         if (entries.length != 2)
         {
            returnNormal(new logical(false));
         }

         returnNormal(new logical(entries[0].equalsIgnoreCase("multipart")));
      }));
   }

   /**
    * Indicates if a given MIME type is json
    *
    * @param _p1 the mime type
    * @return TRUE if the input type is valid and json.
    */
   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "IsJson", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static logical isJson(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);
      ObjectOps.load(MimeTypeHelper.class);

      return function(MimeTypeHelper.class, "IsJson", logical.class, new Block((Body) () ->
      {
         if (p1.isUnknown() || TextOps.isEmpty(p1.getValue()))
         {
            returnNormal(new logical(false));
         }

         String[] entries = splitType(p1.getValue(), -1);

         if (entries.length != 2)
         {
            returnNormal(new logical(false));
         }

         returnNormal(new logical(entries[0].equalsIgnoreCase("application")
                  && entries[1].equalsIgnoreCase("json")));

      }));
   }

   /**
    * Indicates if a given MIME type is xml
    *
    * @param _p1 the mime type
    * @return TRUE if the input type is valid and xml.
    */
   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "IsXML", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static logical isXml(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);
      ObjectOps.load(MimeTypeHelper.class);

      return function(MimeTypeHelper.class, "IsXML", logical.class, new Block((Body) () ->
      {
         if (p1.isUnknown() || TextOps.isEmpty(p1.getValue()))
         {
            returnNormal(new logical(false));
         }

         String[] entries = splitType(p1.getValue(), -1);

         if (entries.length != 2)
         {
            returnNormal(new logical(false));
         }

         if (entries[0].equalsIgnoreCase("application"))
            returnNormal(new logical(entries[1].equalsIgnoreCase("xml")
                     || entries[1].equalsIgnoreCase("xml-external-parsed-entity")
                     || entries[1].equalsIgnoreCase("xml-dtd")));

         returnNormal(new logical(entries[0].equalsIgnoreCase("text")
                  && entries[1].equalsIgnoreCase("xml-external-parsed-entity")));

      }));
   }

   static String[] splitType(final String p1, int numEntries)
   {
      String[] entries = TextOps.entries(p1, "/", numEntries);

      int pos = entries[1].indexOf(";");

      if (pos != -1)
         entries[1] = entries[1].substring(0, pos);

      pos = entries[1].indexOf("+");

      if (pos != -1)
         entries[1] = entries[1].substring(pos + 1);

      entries[1] = entries[1].trim();

      return entries;
   }

}
