/*
** Module   : ClientLibraryBuilder.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 CA  20190526 First version, stubs taken by converting the skeleton using FWD.
** 002 CA  20191024 Added method support levels and updated the class support level.
** 003 ME  20200410 Add stub for build method.
** 004 CA  20200503 Added stub for ClientLibraryBuilder:SslVerifyHost and Library getter.
** 005 MP  20200525 Populate methods to get full working class
** 006 MP  20210113 Add SupportLevels and test as OE12.2
**     ME  20210208 Fix static registry property initialization.
**         20210210 Lazy instantiate registry, the object seems to be deleted otherwise.
** 007 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
**     ME  20210301 Added class 'referent' for static methods.
** 008 ME  20210519 Fix registry lazy loading.
**     ME  20210929 Fix return object on build.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.lib;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.oo.core.Assert;
import com.goldencode.p2j.oo.core.ISupportInitialize;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.oo.logging.IlogWriter;
import com.goldencode.p2j.oo.net.http.*;
import com.goldencode.p2j.oo.net.http.lib.sockets.*;
import com.goldencode.p2j.security.*;
import com.goldencode.p2j.oo.json.objectmodel.JsonArray;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.InternalEntry.Type;

/**
 * Business logic (converted to Java from the 4GL source code
 * in skeleton/oo4gl/OpenEdge/Net/HTTP/Lib/ClientLibraryBuilder.cls).
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Lib.ClientLibraryBuilder")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
public abstract class ClientLibraryBuilder extends ConfigBuilder
{
   @LegacySignature(type = Type.PROPERTY, name = "PROP_SSL_CIPHERS")
   private final character propSslCiphers = TypeFactory.character("sslciphers");

   @LegacySignature(type = Type.PROPERTY, name = "PROP_SSL_HOSTVERIFY")
   private final character propSslHostverify = TypeFactory.character("sslhostverify");

   @LegacySignature(type = Type.PROPERTY, name = "PROP_SSL_PROTOCOLS")
   private final character propSslProtocols = TypeFactory.character("sslprotocols");

   @LegacySignature(type = Type.PROPERTY, name = "PROP_REUSESESSION")
   private final character propReusesession = TypeFactory.character("reusesession");

   @LegacySignature(type = Type.PROPERTY, name = "PROP_SERVERNAME")
   private final character propServername = TypeFactory.character("servername");

   protected static final String CLIENTLIBRARYBUILDER = "OpenEdge.Net.HTTP.Lib.ClientLibraryBuilder";

   /** Builder registry */
   private static ContextLocal<object<? extends BuilderRegistry>> REGISTRY = 
            new ContextLocal<object<? extends BuilderRegistry>>()
   {
      /**
       * Create initial value.
       */
      @Override
      protected object<? extends BuilderRegistry> initialValue()
      {
         return TypeFactory.object(BuilderRegistry.class);
      }
   };

   /**
    * Execute method 
    */
   public void __net_http_lib_ClientLibraryBuilder_execute__()
   {
      externalProcedure(ClientLibraryBuilder.this, new Block((Body) () ->
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
         }
      }));
   }

   /**
    * Constructor
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public void __net_http_lib_ClientLibraryBuilder_constructor__()
   {
      internalProcedure(this, "__net_http_lib_ClientLibraryBuilder_constructor__",
               new Block((Body) () ->
               {
                  __lang_BaseObject_constructor__();
               }));
   }
   
   /**
    * Static Constructor
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static void __net_http_lib_ClientLibraryBuilder_constructor__static__()
   {
      externalProcedure(ClientLibraryBuilder.class, new Block((Body) () ->
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
      }));
   }


   private static void initializeRegistry(final object<? extends BuilderRegistry> _poRegistry)
   {
      internalProcedure(ClientLibraryBuilder.class, "InitializeRegistry", new Block((Body) () ->
      {
         _poRegistry.ref()._put(ObjectOps.getLegacyName(IHttpClientLibrary.class),
                  ObjectOps.getLegacyClass(LegacySocketLibrary.class));
         _poRegistry.ref()._put(ObjectOps.getLegacyName(ClientLibraryBuilder.class),
                  ObjectOps.getLegacyClass(LegacySocketLibraryBuilder.class));
      }));
   }

   /**
    * Get registry.
    * 
    * @return registry.
    */
   @LegacySignature(returns = "OBJECT", qualified = "OpenEdge.Net.HTTP.BuilderRegistry", type = Type.GETTER, name = "Registry")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static object<? extends BuilderRegistry> getRegistry()
   {
      return function(ClientLibraryBuilder.class, "Registry", object.class, new Block((Body) () ->
      {
         if (!REGISTRY.get()._isValid()) 
         {
            REGISTRY.get().assign(ObjectOps.newInstance(BuilderRegistry.class));
            initializeRegistry(REGISTRY.get());
         }
         returnNormal(REGISTRY.get());
      }));
   }

   /**
    * Add a SSLCipher for this library
    * 
    * @param _p1 The SslCipher name to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "AddSslCipher", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> addSslCipher(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "AddSslCipher", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("SSL cipher"));
                  
                  if (!super.hasOption(propSslCiphers).booleanValue())
                     super.setOption(propSslCiphers, ObjectOps.newInstance(JsonArray.class));
                  
                  object<? extends JsonArray> jsonList = (object<? extends JsonArray>) super.getOptionObjectValue(propSslCiphers);
                  
                  jsonList.ref().add(p1);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Add a SSLProtocol for this library
    * 
    * @param _p1 The SslProtocol name to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "AddSslProtocol", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> addSslProtocol(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "AddSslProtocol", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("SSL protocols"));
                  
                  if (!super.hasOption(propSslProtocols).booleanValue())
                     super.setOption(propSslProtocols, ObjectOps.newInstance(JsonArray.class));
                  
                  object<? extends JsonArray> jsonList = (object<? extends JsonArray>) super.getOptionObjectValue(propSslProtocols);
                  
                  jsonList.ref().add(p1);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
   * Builder director method.
   * 
   * @return A builder for the requested type.
   */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Build", qualified = "OpenEdge.Net.HTTP.Lib.ClientLibraryBuilder")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public static object<? extends ClientLibraryBuilder> build()
   {
      object<? extends ClientLibraryBuilder> builder = TypeFactory.object(ClientLibraryBuilder.class);
      
      return function(ClientLibraryBuilder.class, "Build", object.class, new Block((Body) () ->
      {
         object<? extends LegacyClass> builderType = getRegistry().ref().get(new character(CLIENTLIBRARYBUILDER));
         Assert.isType(builderType, ObjectOps.getLegacyClass(ClientLibraryBuilder.class));

         builder.assign(ObjectOps.newInstance(builderType.ref().getType()));

         if (!builder.isUnknown() && builder.ref() instanceof ISupportInitialize)
            ((ISupportInitialize) builder.ref()).initialize();

         returnNormal(builder);

      }));
   }

   /**
    * Add a Option type of character for this library
    * 
    * @param _p1 option name to add.
    * @param _p2 option value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Option", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> option(final character _p1, final character _p2)
   {
      character p1 = TypeFactory.initInput(_p1);
      character p2 = TypeFactory.initInput(_p2);

      return function(ClientLibraryBuilder.class, this, "Option", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("Option name"));
                  
                  super.setOption(p1, p2);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Add a Option type of decimal for this library
    * 
    * @param _p1 option name to add.
    * @param _p2 option value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Option", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "DECIMAL", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> option(final character _p1, final decimal _p2)
   {
      character p1 = TypeFactory.initInput(_p1);
      decimal p2 = TypeFactory.initInput(_p2);

      return function(ClientLibraryBuilder.class, this, "Option", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("Option name"));
                  
                  super.setOption(p1, p2);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Add a Option type of int64 for this library
    * 
    * @param _p1 option name to add.
    * @param _p2 option value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Option", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "INT64", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> option(final character _p1, final int64 _p2)
   {
      character p1 = TypeFactory.initInput(_p1);
      int64 p2 = TypeFactory.initInput(_p2);
      decimal p3 = TypeFactory.decimal();

      return function(ClientLibraryBuilder.class, this, "Option", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("Option name"));
                  
                  p3.assign(p2);
                  
                  super.setOption(p1, p3);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Add a Option type of logical for this library
    * 
    * @param _p1 option name to add.
    * @param _p2 option value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Option", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "LOGICAL", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> option(final character _p1, final logical _p2)
   {
      character p1 = TypeFactory.initInput(_p1);
      logical p2 = TypeFactory.initInput(_p2);

      return function(ClientLibraryBuilder.class, this, "Option", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("Option name"));
                  
                  super.setOption(p1, p2);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Add a Option type of Object for this library
    * 
    * @param _p1 option name to add.
    * @param _p2 option value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "Option", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT"),
            @LegacyParameter(name = "p2", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> option(final character _p1,
            final object<? extends _BaseObject_> _p2)
   {
      character p1 = TypeFactory.initInput(_p1);
      object<? extends _BaseObject_> p2 = TypeFactory.initInput(_p2);

      return function(ClientLibraryBuilder.class, this, "Option", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("Option name"));
                  
                  super.setOption(p1, p2);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Set ReuseSession as option for this library
    * 
    * @param _p1 option logical to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "ReuseSession", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "LOGICAL", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> reuseSession(final logical _p1)
   {
      logical p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "ReuseSession", object.class,
               new Block((Body) () ->
               {
                  Assert.notUnknown(p1, new character("Session reuse flag"));
                  
                  super.setOption(propReusesession, p1);

                  returnNormal(ObjectOps.thisObject());
               }));
   }
   
   /**
    * Set ServerNameIndicator as option for this library
    * 
    * @param _p1 ServerNameIndicator name to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "ServerNameIndicator", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> serverNameIndicator(final character _p1)
   {
      character p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "ServerNameIndicator", object.class,
               new Block((Body) () ->
               {
                  Assert.notNullOrEmpty(p1, new character("Server name"));
                  
                  super.setOption(propServername, p1);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Set SetRequestTimeout as option for this library
    * 
    * @param _p1 SetRequestTimeout value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "SetRequestTimeout", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "DECIMAL", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> setRequestTimeout(final decimal _p1)
   {
      decimal p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "SetRequestTimeout", object.class,
               new Block((Body) () ->
               {
                  Assert.isZeroOrPositive(p1, new character("Request Timeout"));
                  
                  super.setOption(new character("requestTimeout"), p1);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Set SslCiphers as JsonArray object for this library
    * 
    * @param _p1 SslCiphers extent value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "SetSslCiphers", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", extent = -1, mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> setSslCiphers(final character[] _p1)
   {
      character[] p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "SetSslCiphers", object.class,
               new Block((Body) () ->
               {

                  object<? extends JsonArray> arr = ObjectOps.newInstance(JsonArray.class);
                  arr.ref().add(p1);
                                    
                  super.setOption(propSslCiphers, arr);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Set SslProtocols as JsonArray object for this library
    * 
    * @param _p1 SslProtocols extent value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "SetSslProtocols", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "CHARACTER", extent = -1, mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> setSslProtocols(final character[] _p1)
   {
      character[] p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "SetSslProtocols", object.class,
               new Block((Body) () ->
               {
                  object<? extends JsonArray> arr = ObjectOps.newInstance(JsonArray.class);
                  arr.ref().add(p1);
                  
                  super.setOption(propSslProtocols, arr);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Set VerifyHost as logical for this library
    * 
    * @param _p1 VerifyHost logical value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "SslVerifyHost", 
            qualified = "OpenEdge.Net.HTTP.Lib.ClientLibraryBuilder", parameters = {
            @LegacyParameter(name = "p1", mode = "INPUT", type = "LOGICAL") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> sslVerifyHost(
            final logical _p1)
   {
      logical p1 = TypeFactory.initInput(_p1);
      return function(ClientLibraryBuilder.class, this, "SslVerifyHost", object.class,
               new Block((Body) () ->
               {
                  Assert.notUnknown(p1, new character("SSL host verification"));
                  
                  super.setOption(propSslHostverify, p1);

                  returnNormal(ObjectOps.thisObject());
               }));
   }

   /**
    * Set Logger as object for this library
    * 
    * @param _p1 Logger object value to add.
    * 
    * @return <code>ClientLibraryBuilder</code>
    */
   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "UsingLogger", 
            qualified = "openedge.net.http.lib.clientlibrarybuilder", parameters = {
            @LegacyParameter(name = "p1", type = "OBJECT", 
                     qualified = "openedge.logging.ilogwriter", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public object<? extends ClientLibraryBuilder> usingLogger(
            final object<? extends IlogWriter> _p1)
   {
      object<? extends IlogWriter> p1 = TypeFactory.initInput(_p1);

      return function(ClientLibraryBuilder.class, this, "UsingLogger", object.class,
               new Block((Body) () ->
               {
                  Assert.notNull(p1, new character("Log writer"));
                  
                  super.setOption(new character("openedge.logging.ilogwriter"), p1);

                  returnNormal(ObjectOps.thisObject());
               }));
   }
   
   /**
    * Get 'library' attribute value.
    * 
    * @return 'library' attribute value.
    */
   @LegacySignature(returns = "OBJECT", qualified = "OpenEdge.Net.HTTP.IHttpClientLibrary", type = Type.GETTER, name = "Library")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   public abstract object<? extends IHttpClientLibrary> getLibrary();

   /**
    * Get 'PROP_SSL_CIPHERS' attribute value.
    * 
    * @return 'PROP_SSL_CIPHERS' attribute value.
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "PROP_SSL_CIPHERS")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected character getPropSslCiphers()
   {
      return function(ClientLibraryBuilder.class, this, "PROP_SSL_CIPHERS", character.class,
               new Block((Body) () ->
               {
                  returnNormal(propSslCiphers);
               }));
   }
   
   /**
    * Get 'PROP_SSL_HOSTVERIFY' attribute value.
    * 
    * @return 'PROP_SSL_HOSTVERIFY' attribute value.
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "PROP_SSL_HOSTVERIFY")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected character getPropSslHostverify()
   {
      return function(ClientLibraryBuilder.class, this, "PROP_SSL_HOSTVERIFY", character.class,
               new Block((Body) () ->
               {
                  returnNormal(propSslHostverify);
               }));
   }

   /**
    * Get 'PROP_SSL_PROTOCOLS' attribute value.
    * 
    * @return 'PROP_SSL_PROTOCOLS' attribute value.
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "PROP_SSL_PROTOCOLS")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected character getPropSslProtocols()
   {
      return function(ClientLibraryBuilder.class, this, "PROP_SSL_PROTOCOLS", character.class,
               new Block((Body) () ->
               {
                  returnNormal(propSslProtocols);
               }));
   }

   /**
    * Get 'PROP_REUSESESSION' attribute value.
    * 
    * @return 'PROP_REUSESESSION' attribute value.
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "PROP_REUSESESSION")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected character getPropReusesession()
   {
      return function(ClientLibraryBuilder.class, this, "PROP_REUSESESSION", character.class,
               new Block((Body) () ->
               {
                  returnNormal(propReusesession);
               }));
   }

   /**
    * Get 'PROP_SERVERNAME' attribute value.
    * 
    * @return 'PROP_SERVERNAME' attribute value.
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "PROP_SERVERNAME")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   protected character getPropServername()
   {
      return function(ClientLibraryBuilder.class, this, "PROP_SERVERNAME", character.class,
               new Block((Body) () ->
               {
                  returnNormal(propServername);
               }));
   }
}
