/*
** Module   : BinaryBodyWriter.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 IAS  20190923 First version.
** 002 MP   20200707 Add code in methods to get similar behavior like legacy class
** 003 ME   20201207 Refactor write method taking object as input, fix assert error.
** 004 CA   20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                   signature.
**     VVT  20210917 Javadoc fixed. 
**     ME   20210929 Fix function block for getTransferEncoding.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.filter.payload;

import static com.goldencode.p2j.util.BlockManager.externalProcedure;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.function;
import static com.goldencode.p2j.util.BlockManager.internalProcedure;
import static com.goldencode.p2j.util.BlockManager.onBlockLevel;
import static com.goldencode.p2j.util.BlockManager.returnNormal;

import com.goldencode.p2j.oo.common.support.IcharacterHolder;
import com.goldencode.p2j.oo.common.support.IlongcharHolder;
import com.goldencode.p2j.oo.common.support.ImemptrHolder;
import com.goldencode.p2j.oo.core.*;
import com.goldencode.p2j.oo.io.*;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.oo.net.*;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.util.InternalEntry.*;

/**
 * 
 * Create binary body
 *
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Filter.Payload.BinaryBodyWriter")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class BinaryBodyWriter
extends MessageWriter
implements ISupportTransferEncoding
{
   /** transfer encoding */
   private character transferEncoding = TypeFactory.character();

   /**
    * Get transfer encoding
    *  
    * @return transfer encoding
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "TransferEncoding")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public character getTransferEncoding()
   {
      return function(this, "TransferEncoding", character.class, new Block((Body) () -> 
      {
         returnNormal(transferEncoding);
      }));
   }
   
   /**
    * Set transfer encoding
    * 
    * @param _pcEncoding transfer encoding
    */
   @LegacySignature(type = Type.SETTER, name = "TransferEncoding", parameters =
   {
         @LegacyParameter(name = "pcEncoding", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void setTransferEncoding(final character _pcEncoding)
   {
      character pcEncoding = TypeFactory.initInput(_pcEncoding);
      internalProcedure(this, "TransferEncoding", new Block((Body) () -> 
      {
         this.transferEncoding.assign(pcEncoding);
      }));
   }

   /**
    * Execute method
    */
   public void __net_http_filter_payload_BinaryBodyWriter_execute__()
   {
      externalProcedure(BinaryBodyWriter.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
         }
      }));
   }

   /**
    * Constructor
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_http_filter_payload_BinaryBodyWriter_constructor__()
   {
      internalProcedure(this, "__net_http_filter_payload_BinaryBodyWriter_constructor__", 
            new Block((Body) () -> 
      {
         __net_http_filter_payload_MessageWriter_constructor__(
               ObjectOps.getLegacyClass(ByteBucket.class)
         );
      }));
   }

   /**
    * Opens the writer for output.
    */
   @LegacySignature(type = Type.METHOD, name = "Open")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void open() 
   {
      internalProcedure(this, "Open", new Block((Body) () -> 
      {
         if (!entity._isValid())
         {
            entity.assign(ByteBucket.instance());
         }
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "LONGCHAR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(final longchar _pData)
   {
      longchar pData = TypeFactory.initInput(_pData);
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         bucket().putString(pData);
         returnNormal(new int64(TextOps.byteLength(pData)));
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "MEMPTR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(final memptr _pData)
   {
      memptr pData = TypeFactory.initInput(_pData);
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         bucket().putBytes(pData.getPointerValue(), pData.length());
         returnNormal(pData.length());
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = {
            @LegacyParameter(name = "poData", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT") })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL | RT_LVL_FULL)
   @Override
   public int64 write(final object<? extends _BaseObject_> _poData)
   {
      object<? extends _BaseObject_> poData = TypeFactory.initInput(_poData);
      int64 bytesWriten = TypeFactory.int64();
      memptr nPtr = TypeFactory.memptr();
      longchar lChar = TypeFactory.longchar();

      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         this.transferEncoding.assign("binary");

         if (!poData._isValid())
         {
            returnNormal(new int64(0));
         }

         if (ObjectOps.typeOf(poData, Memptr.class).booleanValue() || ObjectOps.typeOf(poData, ImemptrHolder.class).booleanValue())
         {
            try
            {
               nPtr.assign(ObjectOps.typeOf(poData, Memptr.class).booleanValue() ? ObjectOps.cast(poData, Memptr.class).ref().getValue() : ObjectOps.cast(poData, ImemptrHolder.class).ref().getValue());
               returnNormal(write(nPtr));
            }
            finally
            {
               if (nPtr.lengthOf() > 0)
                  nPtr.setLength(0);
            }
         }
         else if (ObjectOps.typeOf(poData, ByteBucket.class).booleanValue())
         {
            object<? extends ByteBucket> obucket = ObjectOps.cast(poData, ByteBucket.class);
            bucket().putBytes(obucket);
         }
         else if (ObjectOps.typeOf(poData, LegacyString.class).booleanValue() ||
                  ObjectOps.typeOf(poData, IlongcharHolder.class).booleanValue() ||
                  ObjectOps.typeOf(poData, IcharacterHolder.class).booleanValue())
         {
            if (ObjectOps.typeOf(poData, LegacyString.class).booleanValue())
               lChar.assign( ObjectOps.cast(poData, LegacyString.class).ref().getValue());
            else if (ObjectOps.typeOf(poData, IlongcharHolder.class).booleanValue())
               lChar.assign( ObjectOps.cast(poData, IlongcharHolder.class).ref().getValue());
            else
               lChar.assign( ObjectOps.cast(poData, IcharacterHolder.class).ref().getValue());
            
            bucket().putString(lChar);
            
            returnNormal(TextOps.byteLength(lChar));
         }
         else if (ObjectOps.typeOf(poData, FileInputStream.class).booleanValue())
         {
            returnNormal(writeFileStream(ObjectOps.cast(poData, FileInputStream.class)));
         } 
         else {
            unsupported(poData);
         }
         
         returnNormal(bytesWriten);
      }));
   }
}
