/*
** Module   : BinaryEntityWriter.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 IAS  20190923 First version.
** 002 MP   20210106 Test as of OE12.2.
**     ME   20210111 Use notNull method on Assert (null mixed with unknown for various data types).
** 003 CA   20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                   signature.
**     ME   20210929 Fix function block for getTransferEncoding.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.filter.payload;

import static com.goldencode.p2j.util.BlockManager.externalProcedure;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.function;
import static com.goldencode.p2j.util.BlockManager.internalProcedure;
import static com.goldencode.p2j.util.BlockManager.onBlockLevel;
import static com.goldencode.p2j.util.BlockManager.returnNormal;

import com.goldencode.p2j.oo.core.*;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.oo.net.*;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.util.InternalEntry.*;

/**
 * 
 * Parse binary payload
 *
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Filter.Payload.BinaryEntityWriter")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class BinaryEntityWriter
extends MessageWriter
implements ISupportTransferEncoding
{
   /** transfer encoding */
   private character transferEncoding = TypeFactory.character();

   /**
    * Get transfer encoding
    *  
    * @return transfer encoding
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "TransferEncoding")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public character getTransferEncoding()
   {
      return function(this, "TransferEncoding", character.class, new Block((Body) () -> 
      {
         returnNormal(transferEncoding);
      }));
   }
   
   /**
    * Set transfer encoding
    * 
    * @param _pcEncoding transfer encoding
    */
   @LegacySignature(type = Type.SETTER, name = "TransferEncoding", parameters =
   {
         @LegacyParameter(name = "pcEncoding", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void setTransferEncoding(character _pcEncoding)
   {
      character pcEncoding = TypeFactory.initInput(_pcEncoding);
      internalProcedure(this, "TransferEncoding", new Block((Body) () -> 
      {
         this.transferEncoding.assign(pcEncoding);
      }));
   }

   /**
    * Execute method
    */
   public void __net_http_filter_payload_BinaryEntityWriter_execute__()
   {
      externalProcedure(BinaryEntityWriter.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
         }
      }));
   }

   /**
    * Constructor
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_http_filter_payload_BinaryEntityWriter_constructor__()
   {
      internalProcedure(this, "__net_http_filter_payload_BinaryEntityWriter_constructor__", 
            new Block((Body) () -> 
      {
         __net_http_filter_payload_MessageWriter_constructor__(
               ObjectOps.getLegacyClass(ByteBucket.class)
         );
      }));
   }

   /**
    * Opens the writer for output.
    */
   @LegacySignature(type = Type.METHOD, name = "Open")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void open() 
   {
      internalProcedure(this, "Open", new Block((Body) () -> 
      {
         if (!entity.isValid().booleanValue())
         {
            entity.assign(ByteBucket.instance());
         }
         
         super.open();
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "LONGCHAR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(longchar _pData)
   {
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         Assert.notNull(_pData, new character("Data"));
         writeInternal(_pData);
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "character", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(character _pData)
   {
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         Assert.notNull(_pData, new character("Data"));
         
         writeInternal(_pData);
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "MEMPTR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(memptr _pData)
   {
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         int64 len = _pData.length();
         Assert.isZeroOrPositive(len, new character("Data size"));
         if (len.longValue() > 0)
         {
            bucket().putBytes(_pData.getPointerValue(), len);
         }
         returnNormal(len);
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "poData", type = "OBJECT",
            qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(object<? extends _BaseObject_> poData)
   {
      int64 size = TypeFactory.int64();

      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         Assert.notNull(poData);

         if (ObjectOps.typeOf(poData, Memptr.class).booleanValue())
         {
            object<? extends Memptr> optr = ObjectOps.cast(poData, Memptr.class);
            Memptr ptr = optr.ref();
            size.assign(ptr.getSize());
            bucket().putBytes(ptr.getPointerValue(), ptr.getSize());
         } else if (ObjectOps.typeOf(poData, ByteBucket.class).booleanValue())
         {
            object<? extends ByteBucket> obucket = ObjectOps.cast(poData, ByteBucket.class);
            size.assign(obucket.ref().getSize());
            bucket().putBytes(obucket);
         } else if (ObjectOps.typeOf(poData, LegacyString.class).booleanValue())
         {
            object<? extends LegacyString> ostr = ObjectOps.cast(poData, LegacyString.class);
            size.assign(ostr.ref().getSize());
            bucket().putString(ostr);
         } else {
            unsupported(poData);
         }
         
         returnNormal(size);
      }));
   }

   /**
    * Helper method for data writing
    * 
    * @param pData data to be written
    */
   private void writeInternal(Text pData)
   {
      memptr ptr = TypeFactory.memptr();
      try 
      {
         ptr.setLength(TextOps.length(pData).longValue() + 1);
         ptr.setString(pData, 1);
         returnNormal(write(ptr));
      }
      finally
      {
         ptr.setLength(0);
      }
   }
}
