/*
** Module   : ConnectRequestFilter.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2020-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 MP  20200603 First version, stubs taken by converting the skeleton using FWD.
** 002 MP  20200707 Add code in constructor and methods
** 003 MP  20210106 Improve code based on existing tests - as OE12.2
*
** 004 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.filter.payload;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;

import static com.goldencode.p2j.report.ReportConstants.CVT_LVL_FULL;
import static com.goldencode.p2j.report.ReportConstants.RT_LVL_FULL;
import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.InternalEntry.Type;

import com.goldencode.p2j.oo.core.Assert;
import com.goldencode.p2j.oo.core.ByteBucket;
import com.goldencode.p2j.oo.lang.AppError;
import com.goldencode.p2j.oo.net.http.IhttpRequest;

/**
 * Business logic (converted to Java from the 4GL source code
 * in OpenEdge/Net/HTTP/Filter/Payload/ConnectRequestFilter.cls).
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Filter.Payload.ConnectRequestFilter")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class ConnectRequestFilter
extends com.goldencode.p2j.oo.net.http.filter.payload.MessageWriter
{
  
   public void __net_http_filter_payload_ConnectRequestFilter_execute__()
   {
      externalProcedure(ConnectRequestFilter.class, ConnectRequestFilter.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
         }
      }));
   }

   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_http_filter_payload_ConnectRequestFilter_constructor__()
   {
      internalProcedure(ConnectRequestFilter.class, this, "__net_http_filter_payload_ConnectRequestFilter_constructor__", new Block((Body) () -> 
      {
         __net_http_filter_payload_MessageWriter_constructor__(ObjectOps.getLegacyClass(ByteBucket.class));
      }));
   }

   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(final object<? extends com.goldencode.p2j.oo.lang._BaseObject_> _pData)
   {
      object<? extends com.goldencode.p2j.oo.lang._BaseObject_> pData = TypeFactory.initInput(_pData);
      longchar CRLF = TypeFactory.longchar("~r~n");
      
      return function(ConnectRequestFilter.class, this, "Write", int64.class, new Block((Body) () -> 
      {
        
         Assert.isType(pData, ObjectOps.getLegacyClass(IhttpRequest.class));
         int64 prevSize = bucket().getSize();
         object<? extends IhttpRequest> req = ObjectOps.cast(pData, IhttpRequest.class);
         
         Assert.notNull(req.ref().getUri(), new character("Request URI"));
         character host = new character(req.ref().getUri().ref().getHost());
         integer port = new integer(req.ref().getUri().ref().getPort());
         
         Assert.notNullOrEmpty(host, new character("Host name"));
         if(port.isUnknown())
         {
            String scheme = req.ref().getUri().ref().getScheme().getValue();
            
            if( scheme.equals("http"))
            {
               port.assign(80);
            }
            if( scheme.equals("https"))
            {
               port.assign(443);
            }
         }
         Assert.notNullOrZero(port, new character("Port"));
         if(port.intValue() > 65535)
         {
            undoThrowTopLevel(AppError.newInstance(TextOps.substitute("Port is too large: &1 (max allowed: 65535)", port.intValue()).getValue(), 0));
         }
         character reqUri = new character(TextOps.substitute("&1:&2", host.getValue(), port.intValue()));
         
         //add connect line
         bucket().putString(new longchar(TextOps.substitute("CONNECT &1 &2", reqUri, req.ref().getVersion())));
         bucket().putString(CRLF);
         
         //add headers
         bucket().putString(new longchar(TextOps.substitute("Host: &1", reqUri)));
         bucket().putString(CRLF);
         
         bucket().putString(new longchar(req.ref().getHeader(new character("User-Agent")).ref().toLegacyString()));
         bucket().putString(CRLF);
         
         bucket().putString(new longchar("Connection: keep-alive"));
         bucket().putString(CRLF);
         
         //add final CRLF who indicate end of headers
         bucket().putString(CRLF);
                  
         returnNormal(MathOps.minus(bucket().getSize(), prevSize));
         
      }));
   }
}
