/*
** Module   : MultipartBodyWriter.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------------Description----------------------------------------
** 001 CA  20190526 First version, stubs taken by converting the skeleton using FWD.
** 002 IAS 20190527 Added multipart support.
** 003 CA  20191024 Added method support levels and updated the class support level.
** 004 MP  20210112 Implement as OE12.2
** 005 CA  20210113 Renamed clear() to clear_(), to follow NameConverter's rules.
** 006 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy signature.
**     VVT 20210917 Javadoc fixed.
**     CA  20220120 Do not used TypeFactory.object when the OO reference must not be tracked or registered.
**                  Do not use TypeFactory.object for internal usages, use ObjectVar if the reference must 
**                  be tracked.
**                  All TypeFactory.object variable definitions must be done outside of the top-level block.
**     ME  20210929 Fix function block for getBoundary.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.filter.payload;

import static com.goldencode.p2j.util.BlockManager.externalProcedure;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.function;
import static com.goldencode.p2j.util.BlockManager.internalProcedure;
import static com.goldencode.p2j.util.BlockManager.onBlockLevel;
import static com.goldencode.p2j.util.BlockManager.returnNormal;
import static com.goldencode.p2j.util.BlockManager.undoThrowTopLevel;

import java.util.concurrent.atomic.*;

import com.goldencode.p2j.oo.core.*;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.oo.net.*;
import com.goldencode.p2j.oo.net.http.*;
import com.goldencode.p2j.oo.net.http.filter.writer.*;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.util.InternalEntry.*;


/**
 * 
 * Write multipart/* body
 *
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Filter.Payload.MultipartBodyWriter")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class MultipartBodyWriter 
extends MessageWriter
implements ISupportMultipartEntity
{
   private static final String CRLF = "\r\n";
   /** Multipart boundary */
   private character boundary = TypeFactory.character();
   /** 'external entity' flag */
   private logical externalEntity = TypeFactory.logical();

   /**
    * Get multipart boundary.
    * 
    * @return boundary. 
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "Boundary")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public character getBoundary()
   {
      return function(this, "Boundary", character.class, new Block((Body) () -> 
      {
         returnNormal(boundary);
      }));
   }
   
   /**
    * Set multipart boundary.
    * 
    * @param boundary multipart boundary/
    */
   @LegacySignature(type = Type.SETTER, name = "Boundary", parameters =
   {
         @LegacyParameter(name = "boundary", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void setBoundary(character boundary)
   {
      internalProcedure(this, "Boundary", new Block((Body) () -> 
      {
         this.boundary.assign(boundary);
      }));
   }

   /**
    * Execute method
    */
   public void __net_http_filter_payload_MultipartBodyWriter_execute__()
   {
      externalProcedure(MultipartBodyWriter.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
         }
      }));
   }

   /**
    * Constructor
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_http_filter_payload_MultipartBodyWriter_constructor__()
   {
      internalProcedure(this, "__net_http_filter_payload_MultipartBodyWriter_constructor__", 
            new Block((Body) () -> 
      {
         __net_http_filter_payload_MessageWriter_constructor__(
               ObjectOps.getLegacyClass(ByteBucket.class));
         setBoundary(SecurityOps.convertUniversalUIDToGlobalUID());
      }));
   }

   /**
    * Opens the writer for output.
    */
   @LegacySignature(type = Type.METHOD, name = "Open")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void open() 
   {
      internalProcedure(this, "Open", new Block((Body) () -> 
      {
         this.externalEntity.assign(entity.isValid());
         if (!externalEntity.booleanValue())
         {
            entity.assign(ByteBucket.instance());
         }
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "poData", type = "OBJECT",
            qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(object<? extends _BaseObject_> _poData)
   {
      object<? extends _BaseObject_> poData = TypeFactory.initInput(_poData);
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         
         if (!poData._isValid())
            returnNormal(0);
         
         if (!ObjectOps.typeOf(poData, MultipartEntity.class).booleanValue())
         {
            String errMsg = "Unsupported object type: " + poData.ref().
                     getLegacyClass().ref().getTypeName().getValue();
            undoThrowTopLevel(AppError.newInstance(errMsg, 0));
         }
                  
         returnNormal(write_1(ObjectOps.cast(poData, MultipartEntity.class)));
      }));
   }
   
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "OBJECT", qualified = "openedge.net.multipartentity", mode = "INPUT")
   })
   protected int64 write_1(final object<? extends MultipartEntity> _pData)
   {
      object<? extends MultipartEntity> pData = TypeFactory.initInput(_pData);
      object<? extends MessagePart> mpart = TypeFactory.object(MessagePart.class);
      object<? extends MessageWriter> writer = TypeFactory.object(MessageWriter.class);
      object<? extends ByteBucket> part = TypeFactory.object(ByteBucket.class);
      object[][] headers = { TypeFactory.objectExtent(HttpHeader.class, false) };

      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         Assert.notNull(pData, new character("Multipart entity"));
                  
         long initialSize = bucket().getSize().longValue();
         
         this.boundary.assign(pData.ref().getBoundary());
         
         part.assign(ObjectOps.newInstance(ByteBucket.class));
         
         if (!TextOps.isEmpty(pData.ref().getPrologue().getValue()))
         {
            bucket().putString(new longchar(pData.ref().getPrologue()));
            bucket().putString(new longchar(CRLF));
         }
         
         for (int ip = 0; ip < pData.ref().getSize().intValue(); ip++)
         {
            part.ref().clear_();
            bucket().putString(new longchar(TextOps.concat("--", this.boundary.getValue(), CRLF)));               
            mpart.assign(pData.ref().getPart(new integer(ip+1)));
            writer.assign(BodyWriterBuilder.build(mpart).ref().writeTo(part).ref().getWriter());
            Assert.notNull(writer, new character("Part writer"));
            writer.ref().open();
            writer.ref().write(mpart.ref().getBody());
            writer.ref().close();

            // TODO: add headers
            object<? extends HttpHeaderCollection> cheaders = mpart.ref().getHeaders();
            int nh = cheaders.ref().getAll(
                  new OutputExtentParameter<object<? extends HttpHeader>>()
                  {
                     @Override
                     public object<? extends HttpHeader>[] getVariable()
                     {
                        return headers[0];
                     }

                     @Override
                     public void setVariable(object<? extends HttpHeader>[] reference)
                     {
                        headers[0] = reference;
                     }
                  }                  
                 ).intValue();
            for(int hn = nh; hn > 0; hn--)
            {
               bucket().putString(new longchar(TextOps.concat(headers[0][hn-1].ref().toLegacyString(), CRLF)));                     
            }
            bucket().putString(new longchar(CRLF));
            bucket().putBytes(part);
            bucket().putString(new longchar(CRLF));
         }
         bucket().putString(new longchar(TextOps.concat("--", this.boundary, "--", CRLF)));
         
         if (!TextOps.isEmpty(pData.ref().getEpilogue().getValue()))
         {
            bucket().putString(new longchar(pData.ref().getEpilogue()));
         }
         else if (!this.externalEntity.booleanValue()) 
         {
            bucket().putString(new longchar(CRLF));
         }
         
         returnNormal(new int64(bucket().getSize().longValue() - initialSize));

      }));
   }
   
}
