/*
** Module   : StringEntityWriter.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 CA  20190526 First version, stubs taken by converting the skeleton using FWD.
** 002 CA  20191024 Added method support levels and updated the class support level.
** 003 ME  20200413 Use the ISupportEncoding interface from Net package.
** 004 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
** 005 ME  20210325 Implement/fix functionality for OE12.2.
**     VVT 20210917 Javadoc fixed. 
**     ME  20210929 Fix function block for getEncoding.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.filter.payload;

import static com.goldencode.p2j.util.BlockManager.externalProcedure;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.function;
import static com.goldencode.p2j.util.BlockManager.internalProcedure;
import static com.goldencode.p2j.util.BlockManager.onBlockLevel;
import static com.goldencode.p2j.util.BlockManager.returnNormal;

import com.goldencode.p2j.oo.net.ISupportEncoding;
import com.goldencode.p2j.oo.common.support.IcharacterHolder;
import com.goldencode.p2j.oo.common.support.IlongcharHolder;
import com.goldencode.p2j.oo.common.support.ImemptrHolder;
import com.goldencode.p2j.oo.core.*;
import com.goldencode.p2j.oo.json.objectmodel.*;
import com.goldencode.p2j.oo.lang.*;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.util.InternalEntry.*;

/**
 * 
 * Parse text/* payload
 *
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Filter.Payload.StringEntityWriter")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class StringEntityWriter
extends MessageWriter
implements ISupportEncoding
{

   /** encoding */
   private character encoding = TypeFactory.character();

   /**
    * Get transfer encoding
    * 
    * @return transfer encoding 
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "TransferEncoding")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public character getEncoding()
   {
      return function(this, "Encoding", character.class, new Block((Body) () -> 
      {
         returnNormal(encoding);
      }));
   }
   
   /**
    * Set transfer encoding
    * 
    * @param _pcEncoding transfer encoding
    */
   @LegacySignature(type = Type.SETTER, name = "Logger", parameters =
   {
         @LegacyParameter(name = "pcEncoding", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void setEncoding(character _pcEncoding)
   {
      character pcEncoding = TypeFactory.initInput(_pcEncoding);
      internalProcedure(this, "Encoding", new Block((Body) () -> 
      {
         this.encoding.assign(pcEncoding);
      }));
   }

   /**
    * Execute method.
    */
   public void __net_http_filter_payload_StringEntityWriter_execute__()
   {
      externalProcedure(StringEntityWriter.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
            // no-op
         }
      }));
   }

   /**
    * Constructor.
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_http_filter_payload_StringEntityWriter_constructor__()
   {
      internalProcedure(this, "__net_http_filter_payload_StringEntityWriter_constructor__", 
            new Block((Body) () -> 
      {
         __net_http_filter_payload_MessageWriter_constructor__(
               ObjectOps.getLegacyClass(LegacyString.class)
         );
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "LONGCHAR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_BASIC)
   @Override
   public int64 write(longchar _pData)
   {
      longchar pData = TypeFactory.initInput(_pData);
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         integer iLen = TextOps.byteLength(pData);
         
         // 4GL quirk, why simply not null?
         Assert.isZeroOrPositive(iLen, new character("Data"));
         
         // TODO: fix encoding
         this.entity.assign(ObjectOps.newInstance(LegacyString.class, "I", pData));
         
         returnNormal(iLen);
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(character _pData)
   {
      longchar lc = TypeFactory.longchar();
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         lc.assign(_pData);
         returnNormal(write(lc));
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "MEMPTR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_BASIC)
   @Override
   public int64 write(memptr _pData)
   {
      memptr pData = TypeFactory.initInput(_pData);
      longchar lc = TypeFactory.longchar();
      int64 len = TypeFactory.int64();
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         // TODO: fix encoding
         len.assign(pData.length());
         Assert.isZeroOrPositive(len, new character("Data size"));
         
         if (len.longValue() > 0)
         {
            new LobCopy(new SourceLob(pData), new TargetLob(lc)).run();
         }
         
         this.entity.assign(ObjectOps.newInstance(LegacyString.class, "I", lc));
         
         returnNormal(len);
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "poData", type = "OBJECT",
            qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(object<? extends _BaseObject_> _poData)
   {
      object<? extends _BaseObject_> poData = TypeFactory.initInput(_poData);
      memptr ptr = TypeFactory.memptr();
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         if (!poData._isValid())
         {
            returnNormal(new int64(0));
         }
         else if (ObjectOps.typeOf(poData, JsonObject.class).booleanValue())
         {
            object<? extends JsonObject> json = ObjectOps.cast(poData, JsonObject.class);
            this.encoding.assign(new character("UTF-8"));
            returnNormal(write(json.ref().getJsonText()));
         }
         else if (ObjectOps.typeOf(poData, JsonArray.class).booleanValue())
         {
            object<? extends JsonArray> json = ObjectOps.cast(poData, JsonArray.class);
            this.encoding.assign(new character("UTF-8"));
            returnNormal(write(json.ref().getJsonText()));
         }
         else if (ObjectOps.typeOf(poData, IlongcharHolder.class).booleanValue())
         {
            object<? extends IlongcharHolder> lc = ObjectOps.cast(poData, IlongcharHolder.class);
            returnNormal(write(lc.ref().getValue()));
         }
         else if (ObjectOps.typeOf(poData, IcharacterHolder.class).booleanValue())
         {
            object<? extends IcharacterHolder> lc = ObjectOps.cast(poData, IcharacterHolder.class);
            returnNormal(write(lc.ref().getValue()));
         }
         else if (ObjectOps.typeOf(poData, ImemptrHolder.class).booleanValue())
         {
            try {
               ptr.assign(ObjectOps.cast(poData, ImemptrHolder.class).ref().getValue());
               returnNormal(write(ptr));
            } finally {
               ptr.setLength(0);
            }
         }
         else if (ObjectOps.typeOf(poData, ByteBucket.class).booleanValue())
         {
            try {
               ptr.assign(ObjectOps.cast(poData, ByteBucket.class).ref().getBytes().ref().getValue());
               returnNormal(write(ptr));
            } finally {
               ptr.setLength(0);
            }
         }
         else {
            returnNormal(write(poData.ref().toLegacyString()));
         }
      }));
   }
}
