/*
** Module   : XmlBodyWriter.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------Description--------------------------------
** 001 CA  20190526 First version, stubs taken by converting the skeleton using FWD.
** 002 CA  20191024 Added method support levels and updated the class support level.
** 003 MP  20200612 Added missing methods as stubs taken by converting the skeleton using FWD.
** 003 MP  20210112 make write methods work as OE 12.2. Still need attention
** 004 ME  20210131 Fix ctor call for TargetData.
**         20210204 Implement missing methods, fix write handle/object.
** 005 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy
**                  signature.
**     VVT 20210917 Javadoc fixed. 
**     ME  20210929 Fix function block for getEncoding, fix error message on write.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.filter.payload;

import static com.goldencode.p2j.util.BlockManager.externalProcedure;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.function;
import static com.goldencode.p2j.util.BlockManager.internalProcedure;
import static com.goldencode.p2j.util.BlockManager.onBlockLevel;
import static com.goldencode.p2j.util.BlockManager.returnNormal;
import static com.goldencode.p2j.util.BlockManager.undoThrow;

import com.goldencode.p2j.oo.common.support.IcharacterHolder;
import com.goldencode.p2j.oo.common.support.IlongcharHolder;
import com.goldencode.p2j.oo.common.support.ImemptrHolder;
import com.goldencode.p2j.oo.core.*;
import com.goldencode.p2j.oo.io.FileInputStream;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.persist.TargetData;
import com.goldencode.p2j.persist.serial.Serializator;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.util.InternalEntry.*;

/**
 * 
 * Write application/xml body
 *
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Filter.Payload.XmlBodyWriter")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class XmlBodyWriter 
extends MessageWriter
implements ISupportEncoding
{

   /** encoding */
   @LegacySignature(type = Type.PROPERTY, name = "Encoding")
   private character encoding = TypeFactory.character();

   /**
    * Get transfer encoding
    * 
    * @return transfer encoding 
    */
   @LegacySignature(returns = "CHARACTER", type = Type.GETTER, name = "TransferEncoding")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public character getEncoding()
   {
      return function(this, "Encoding", character.class, new Block((Body) () -> 
      {
         returnNormal(encoding);
      }));
   }
   
   /**
    * Set transfer encoding
    * 
    * @param _pcEncoding transfer encoding
    */
   @LegacySignature(type = Type.SETTER, name = "Logger", parameters =
   {
         @LegacyParameter(name = "pcEncoding", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void setEncoding(character _pcEncoding)
   {
      character pcEncoding = TypeFactory.initInput(_pcEncoding);
      internalProcedure(this, "Encoding", new Block((Body) () -> 
      {
         this.encoding.assign(pcEncoding);
      }));
   }

   /** 
    * Execute method.
    */
   public void __net_http_filter_payload_XmlBodyWriter_execute__()
   {
      externalProcedure(XmlBodyWriter.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
         }
      }));
   }

   /**
    * Constructor
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_http_filter_payload_XmlBodyWriter_constructor__()
   {
      internalProcedure(this, "__net_http_filter_payload_XmlBodyWriter_constructor__", 
            new Block((Body) () -> 
      {
         __net_http_filter_payload_MessageWriter_constructor__(
               ObjectOps.getLegacyClass(ByteBucket.class)
         );
      }));
   }

   /**
    * Opens the writer for output.
    */
   @LegacySignature(type = Type.METHOD, name = "Open")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public void open() 
   {
      internalProcedure(this, "Open", new Block((Body) () -> 
      {
         if (!entity._isValid())
         {
            entity.assign(ByteBucket.instance());
         }
         
         super.open();
      }));
   }
   
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "HANDLE", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_STUB)
   @Override
   public int64 write(final handle _pData)
   {
      handle pData = TypeFactory.initInput(_pData);
      memptr mResult = TypeFactory.memptr();
      
      return function(XmlBodyWriter.class, this, "Write", int64.class, new Block((Body) () -> 
      {
         if (pData._isValid())
         {
            if (pData.isType("X-DOCUMENT"))
            {
               pData.unwrapXDocument().save("memptr", mResult);
               encoding.assign(ConvCp.toMime(I18nOps.getCPInternal()));
            }
            else if (pData.isType("TEMP-TABLE") || pData.isType("DATASET") || pData.isType("BUFFER"))
            {
               pData.unwrapXmlData().writeXml(new TargetData(Serializator.TYPE_MEMPTR, mResult), SessionUtils.isDebugAlert());
               encoding.assign("utf-8");
            }  
            else
            {
               String errMsg = "Unsupported handle type: " + pData.unwrapType().getResourceType().getValue();
               undoThrow(AppError.newInstance(errMsg, 0));
            }
         }
         try
         {
            bucket().putBytes(mResult);
            returnNormal(mResult.length());
         }
         finally
         {
            mResult.setLength(0);
         }
         
      }));
   }
   
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "LONGCHAR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(final longchar _pData)
   {
      longchar pData = TypeFactory.initInput(_pData);
      longchar mContent = TypeFactory.longchar();
      
      return function(XmlBodyWriter.class, this, "Write", int64.class, new Block((Body) () -> 
      {
         Assert.notNull(pData, new character("XML data"));
         
         mContent.assign(TextOps.trim(pData));
         
         if ( !TextOps.begins(mContent, "<").booleanValue() )
         {
            String errMsg = "Character data does not appear to be XML; starts with \"" + 
                     TextOps.substring(mContent, 1, TextOps.indexOf(mContent, "<")).getValue() + "\"";                     
            undoThrow(AppError.newInstance(errMsg, 0));
         }
         
         encoding.assign(ConvCp.toMime(pData.getCodePage()));
         bucket().putString(pData);
         returnNormal(TextOps.byteLengthOf(pData));
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _pData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "pData", type = "MEMPTR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(final memptr _pData)
   {
      memptr pData = TypeFactory.initInput(_pData);
      character mContent = TypeFactory.character();
      int64 mCheckLength = TypeFactory.int64(10l);
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         if (pData.length().intValue() == 0)
            returnNormal(new int64(0));
         
         if (pData.length().intValue() < 10)
            mCheckLength.assign(pData.length());
         
         mContent.assign(TextOps.trim(pData.getString(1, mCheckLength)));
         
         if ( !TextOps.begins(mContent, "<").booleanValue() )
         {
            String errMsg = "Data does not appear to be XML; starts with \"" + 
                     TextOps.substring(mContent, 1, 10).getValue() + "\"";                     
            undoThrow(AppError.newInstance(errMsg, 0));
         }
         
         encoding.assign(ConvCp.toMime(I18nOps.getCPInternal()));
         bucket().putBytes(pData.getPointerValue(), pData.length());
         returnNormal(pData.length());
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(object<? extends _BaseObject_> _poData)
   {
      object<? extends _BaseObject_> poData = TypeFactory.initInput(_poData);
      
      return function(this, "Write", int64.class, new Block((Body) () -> {
         
         if (!poData._isValid())
            returnNormal(0);
         
         if (ObjectOps.typeOf(poData, WidgetHandle.class).booleanValue())
         {
            returnNormal(this.write(ObjectOps.cast(poData, WidgetHandle.class).ref().getValue()));
         }
         else if (ObjectOps.typeOf(poData, IlongcharHolder.class).booleanValue())
         {
            returnNormal(this.write(ObjectOps.cast(poData, IlongcharHolder.class).ref().getValue()));
         }
         else if (ObjectOps.typeOf(poData, IcharacterHolder.class).booleanValue())
         {
            returnNormal(this.write(new longchar(ObjectOps.cast(poData, IcharacterHolder.class).ref().getValue())));
         }
         else if (ObjectOps.typeOf(poData, ImemptrHolder.class).booleanValue())
         {
            returnNormal(this.write(ObjectOps.cast(poData, ImemptrHolder.class).ref().getValue()));
         }
         else if (ObjectOps.typeOf(poData, FileInputStream.class).booleanValue())
         {
            object<? extends FileInputStream> oFis = ObjectOps.cast(poData, FileInputStream.class);
            
            String[] result = TextOps.splitInt(oFis.ref().getFileName().getValue(), '.');
                        
            if (result[result.length - 1].equalsIgnoreCase("xml") || result[result.length - 1].equalsIgnoreCase("xsd"))
            {
               returnNormal(this.writeFileStream(oFis));
            }
            String errMsg = "Unsupported file extension ." + result[result.length - 1] + " for XML";                     
            undoThrow(AppError.newInstance(errMsg, 0));
           
         }
         
         unsupported(poData);
         
      }));
   }
}
