/*
** Module   : XmlEntityWriter.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2022, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------------Description----------------------------------------
** 001 CA  20190526 First version, stubs taken by converting the skeleton using FWD.
** 002 CA  20191024 Added method support levels and updated the class support level.
** 003 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy signature.
** 004 ME  20210326 Implement as per OE12.2.
**     VVT 20210917 Javadoc fixed. 
**     CA  20220120 Do not used TypeFactory.object when the OO reference must not be tracked or registered.
**                  Do not use TypeFactory.object for internal usages, use ObjectVar if the reference must 
**                  be tracked.
**                  All TypeFactory.object variable definitions must be done outside of the top-level block.
**     CA  20220923 Variable definitions (including associated with parameters) must be done always outside of 
**                  the BlockManager API
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.net.http.filter.payload;

import static com.goldencode.p2j.util.BlockManager.externalProcedure;
import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.BlockManager.function;
import static com.goldencode.p2j.util.BlockManager.internalProcedure;
import static com.goldencode.p2j.util.BlockManager.onBlockLevel;
import static com.goldencode.p2j.util.BlockManager.returnNormal;
import static com.goldencode.p2j.util.BlockManager.undoThrow;

import com.goldencode.p2j.oo.common.support.IcharacterHolder;
import com.goldencode.p2j.oo.common.support.IhandleHolder;
import com.goldencode.p2j.oo.common.support.IlongcharHolder;
import com.goldencode.p2j.oo.common.support.ImemptrHolder;
import com.goldencode.p2j.oo.core.*;
import com.goldencode.p2j.oo.io.FileInputStream;
import com.goldencode.p2j.oo.lang.*;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.util.BlockManager.Action;
import com.goldencode.p2j.util.BlockManager.Condition;
import com.goldencode.p2j.util.InternalEntry.*;
import com.goldencode.p2j.xml.*;

/**
 * 
 * Parse application/xml payload.
 *
 */
@LegacyResource(resource = "OpenEdge.Net.HTTP.Filter.Payload.XmlEntityWriter")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public class XmlEntityWriter
extends MessageWriter
{
   /**
    * Execute.
    */
   public void __net_http_filter_payload_XmlEntityWriter_execute__()
   {
      externalProcedure(XmlEntityWriter.this, new Block((Body) () -> 
      {
         onBlockLevel(Condition.ERROR, Action.THROW);
         {
            // no-op
         }
      }));
   }

   /**
    * Constructor.
    */
   @LegacySignature(type = Type.CONSTRUCTOR)
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void __net_http_filter_payload_XmlEntityWriter_constructor__()
   {
      internalProcedure(this, "__net_http_filter_payload_XmlEntityWriter_constructor__", 
            new Block((Body) () -> 
      {
         __net_http_filter_payload_MessageWriter_constructor__(
               ObjectOps.getLegacyClass(IhandleHolder.class)
         );
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "poData", type = "LONGCHAR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(longchar _poData)
   {
      longchar poData = TypeFactory.initInput(_poData);
      int64 len = TypeFactory.int64();
      handle hdoc = TypeFactory.handle();
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         Assert.notNull(poData, new character("Data"));
         len.assign(TextOps.byteLength(poData));
         XmlFactory.createXDocument(hdoc);
         if (len.longValue() == 0)
         {
            logMessage(new character("Zero-length data received in longchar"), new integer(5));
         }
         else 
         {
            hdoc.unwrapXDocument().load(new character("longchar"), poData, new logical(false));
         }
         this.entity.assign(ObjectOps.newInstance(WidgetHandle.class, "I", hdoc));
         returnNormal(len);
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "poData", type = "CHARACTER", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(character _poData)
   {
      character poData = TypeFactory.initInput(_poData);
      longchar lc = TypeFactory.longchar();
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         lc.assign(poData);
         returnNormal(write(lc));
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "poData", type = "MEMPTR", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(memptr _poData)
   {
      memptr poData = TypeFactory.initInput(_poData);
      int64 len = TypeFactory.int64();
      handle hdoc = TypeFactory.handle();
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         len.assign(poData.length());
         
         XmlFactory.createXDocument(hdoc);
         if (len.longValue() == 0)
         {
            logMessage(new character("Zero-length data received in memptr"), new integer(5));
         }
         else 
         {
            hdoc.unwrapXDocument().load(new character("memptr"), poData, new logical(false));
         }
         this.entity.assign(ObjectOps.newInstance(WidgetHandle.class, "I", hdoc));
         returnNormal(len);
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _phData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "phData", type = "HANDLE", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(handle _phData)
   {
      handle phData = TypeFactory.initInput(_phData);
      handle xDoc = TypeFactory.handle();
      
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         if (!phData._isValid()) 
         {
            returnNormal(0);
         }
         else if (handle.isHandleTypeOf(phData, XDocument.class))
         {
            this.entity.assign(ObjectOps.newInstance(WidgetHandle.class, "I", phData));  
         }
         else if (handle.isHandleTypeOf(phData, XNodeRef.class))
         {
            if (!this.entity._isValid())
            {
               XmlFactory.createXDocument(xDoc);
               this.entity.assign(ObjectOps.newInstance(WidgetHandle.class, "I", xDoc)); 
            } 
            else
            {
               xDoc.assign(ObjectOps.cast(this.entity, IhandleHolder.class).ref().getValue());
            }
            
            xDoc.unwrapXDocument().appendChild(phData);
         }
         else 
         {
            undoThrow(AppError.newInstance(String.format("Unsupported handle type: %s", 
                     phData.unwrapType().getResourceType().toStringMessage()), 0));
            
         }
         
         returnNormal(new int64());
      }));
   }

   /**
    * Write data to/from the entity
    * 
    * @param _poData data to be written
    * 
    * @return number of bytes written
    */
   @LegacySignature(returns = "INT64", type = Type.METHOD, name = "Write", parameters = 
   {
      @LegacyParameter(name = "poData", type = "OBJECT",
            qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   @Override
   public int64 write(object<? extends _BaseObject_> _poData)
   {
      object<? extends _BaseObject_> poData = TypeFactory.initInput(_poData);
      object<? extends Memptr> mptr = TypeFactory.object(Memptr.class);
               
      return function(this, "Write", int64.class, new Block((Body) () -> 
      {
         if (!poData._isValid())
         {
            returnNormal(0);
         }
         if (ObjectOps.typeOf(poData, IlongcharHolder.class).booleanValue())
         {
            returnNormal(write(ObjectOps.cast(poData, IlongcharHolder.class).ref().getValue()));
         }
         if (ObjectOps.typeOf(poData, IcharacterHolder.class).booleanValue())
         {
            returnNormal(write(ObjectOps.cast(poData, IcharacterHolder.class).ref().getValue()));
         }
         if (ObjectOps.typeOf(poData, IhandleHolder.class).booleanValue())
         {
            returnNormal(write(ObjectOps.cast(poData, IhandleHolder.class).ref().getValue()));
         }
         if (ObjectOps.typeOf(poData, ImemptrHolder.class).booleanValue())
         {
            returnNormal(write(ObjectOps.cast(poData, ImemptrHolder.class).ref().getValue()));
         }
         else if (ObjectOps.typeOf(poData, ByteBucket.class).booleanValue()) 
         {
            mptr.assign(ObjectOps.cast(poData, ByteBucket.class).ref().getBytes());
            returnNormal(write(mptr.ref().getValue()));
         }
         else if (ObjectOps.typeOf(poData, FileInputStream.class).booleanValue())
         {
            object<? extends FileInputStream> fis = ObjectOps.cast(poData, FileInputStream.class);
            
            String[] result = TextOps.splitInt(fis.ref().getFileName().getValue(), '.');
            
            if (result[result.length - 1].equalsIgnoreCase("xml") || result[result.length - 1].equalsIgnoreCase("xsd"))
            {
               returnNormal(this.writeFileStream(fis));
            }

            String msg = String.format("Unsupported file extension .%s for XML", result[result.length - 1]);
            undoThrow(AppError.newInstance(msg, 0));
         }
         else 
         {
            unsupported(poData);
         }
      }));
   }
}
